@@ -61,10 +61,13 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.Map;
 import com.openexchange.admin.exceptions.OXGenericException;
 import com.openexchange.admin.reseller.daemons.ClientAdminThreadExtended;
 import com.openexchange.admin.reseller.rmi.OXResellerTools;
+import com.openexchange.admin.reseller.rmi.dataobjects.CustomField;
 import com.openexchange.admin.reseller.rmi.dataobjects.ResellerAdmin;
 import com.openexchange.admin.reseller.rmi.dataobjects.Restriction;
 import com.openexchange.admin.reseller.rmi.exceptions.OXResellerException;
@@ -79,10 +82,12 @@ import com.openexchange.admin.rmi.exceptions.InvalidDataException;
 import com.openexchange.admin.rmi.exceptions.PoolException;
 import com.openexchange.admin.rmi.exceptions.StorageException;
 import com.openexchange.admin.tools.AdminCache;
+import com.openexchange.database.Databases;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.impl.IDGenerator;
 import com.openexchange.groupware.userconfiguration.RdbUserPermissionBitsStorage;
 import com.openexchange.groupware.userconfiguration.UserPermissionBits;
+import com.openexchange.tools.sql.DBUtils;
 
 /**
  * @author choeger
@@ -114,9 +119,13 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         PreparedStatement prep = null;
         ResultSet rs = null;
 
+        log.debug("change admin {}", adm);
+
+        boolean rollback = false;
         try {
             oxcon = cache.getWriteConnectionForConfigDB();
             oxcon.setAutoCommit(false);
+            rollback = true;
 
             final String name = adm.getName();
             int sid = 0;
@@ -188,15 +197,12 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             }
 
             oxcon.commit();
+            rollback = false;
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
-            doRollback(oxcon);
             throw AdminCache.parseDataTruncation(dt);
         } catch (final RuntimeException e) {
             log.error("", e);
-            if (null != oxcon){
-                doRollback(oxcon);
-            }
             throw e;
         } catch (final PoolException e) {
             log.error("", e);
@@ -204,21 +210,20 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final InvalidDataException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final NoSuchAlgorithmException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final UnsupportedEncodingException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(oxcon);
+            }
             cache.closeWriteConfigDBSqlStuff(oxcon, prep, rs);
         }
     }
@@ -228,9 +233,13 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection oxcon = null;
         PreparedStatement prep = null;
 
+        log.debug("create admin {}", adm);
+
+        boolean rollback = false;
         try {
             oxcon = cache.getWriteConnectionForConfigDB();
             oxcon.setAutoCommit(false);
+            rollback = true;
 
             final int adm_id = IDGenerator.getId(oxcon);
 
@@ -260,36 +269,32 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             }
 
             oxcon.commit();
+            rollback = false;
 
             adm.setId(adm_id);
             return adm;
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
-            doRollback(oxcon);
             throw AdminCache.parseDataTruncation(dt);
         } catch (final RuntimeException e) {
             log.error("", e);
-            if (null != oxcon) {
-                doRollback(oxcon);
-            }
             throw e;
         } catch (final PoolException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final NoSuchAlgorithmException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final UnsupportedEncodingException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(oxcon);
+            }
             cache.closeWriteConfigDBSqlStuff(oxcon, prep);
         }
     }
@@ -299,12 +304,16 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection oxcon = null;
         PreparedStatement prep = null;
 
+        log.debug("delete admin {}", adm);
+
+        boolean rollback = false;
         try {
 
             final ResellerAdmin tmp = getData(new ResellerAdmin[] { adm })[0];
 
             oxcon = cache.getWriteConnectionForConfigDB();
             oxcon.setAutoCommit(false);
+            rollback = true;
 
             prep = oxcon.prepareStatement("DELETE FROM subadmin_restrictions WHERE sid=?");
             prep.setInt(1, tmp.getId());
@@ -316,31 +325,32 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             prep.executeUpdate();
 
             oxcon.commit();
+            rollback = false;
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
-            doRollback(oxcon);
             throw AdminCache.parseDataTruncation(dt);
         } catch (final RuntimeException e) {
             log.error("", e);
-            if (null != oxcon) {
-                doRollback(oxcon);
-            }
             throw e;
         } catch (final PoolException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(oxcon);
+            }
             cache.closeWriteConfigDBSqlStuff(oxcon, prep);
         }
     }
 
     @Override
     public ResellerAdmin[] list(final String search_pattern, final int pid) throws StorageException {
+
+        log.debug("list using pattern {}, parent id {}", search_pattern, pid);
+
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
@@ -390,30 +400,36 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
 
     private ResellerAdmin getRestrictionDataForAdmin(final ResellerAdmin admin, final Connection con) throws SQLException {
         final PreparedStatement prep = con.prepareStatement("SELECT subadmin_restrictions.rid,sid,name,value FROM subadmin_restrictions INNER JOIN restrictions ON subadmin_restrictions.rid=restrictions.rid WHERE sid=?");
-        if( admin.getParentId() > 0 ) {
-            prep.setInt(1, admin.getParentId());
-        } else {
-            prep.setInt(1, admin.getId());
-        }
-        final ResultSet rs = prep.executeQuery();
-
-        final HashSet<Restriction> res = new HashSet<Restriction>();
-        while (rs.next()) {
-            final Restriction r = new Restriction();
-            r.setId(rs.getInt(DATABASE_COLUMN_ID));
-            r.setName(rs.getString(DATABASE_COLUMN_NAME));
-            r.setValue(rs.getString(DATABASE_COLUMN_VALUE));
-            if( admin.getParentId() > 0 && r.getName().equals(Restriction.SUBADMIN_CAN_CREATE_SUBADMINS)) {
-                continue;
+        ResultSet rs = null;
+        try {
+            if (admin.getParentId() > 0) {
+                prep.setInt(1, admin.getParentId());
+            } else {
+                prep.setInt(1, admin.getId());
             }
-            res.add(r);
+
+            rs = prep.executeQuery();
+
+            final HashSet<Restriction> res = new HashSet<Restriction>();
+            while (rs.next()) {
+                final Restriction r = new Restriction();
+                r.setId(rs.getInt(DATABASE_COLUMN_ID));
+                r.setName(rs.getString(DATABASE_COLUMN_NAME));
+                r.setValue(rs.getString(DATABASE_COLUMN_VALUE));
+                if (admin.getParentId() > 0 && r.getName().equals(Restriction.SUBADMIN_CAN_CREATE_SUBADMINS)) {
+                    continue;
+                }
+                res.add(r);
+            }
+            if (res.size() > 0) {
+                admin.setRestrictions(res.toArray(new Restriction[res.size()]));
+            }
+            rs.close();
+            prep.close();
+            return admin;
+        } finally {
+            DBUtils.closeSQLStuff(rs, prep);
         }
-        if (res.size() > 0) {
-            admin.setRestrictions(res.toArray(new Restriction[res.size()]));
-        }
-        rs.close();
-        prep.close();
-        return admin;
     }
 
     @Override
@@ -423,6 +439,8 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
 
     @Override
     public ResellerAdmin[] getData(final ResellerAdmin[] admins, final int pid) throws StorageException {
+        log.debug("getData");
+
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
@@ -503,6 +521,8 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
     }
 
     private boolean existsAdmin(final ResellerAdmin[] admins, final int pid) throws StorageException {
+        log.debug("existsAdmin");
+
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
@@ -562,29 +582,56 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
     }
 
     @Override
-    public void ownContextToAdmin(final Context ctx, final Credentials creds) throws StorageException {
+    public void restore(Context ctx, int subadmin, Restriction[] restrictions, CustomField[] customFields) throws StorageException {
         Connection oxcon = null;
         PreparedStatement prep = null;
+
+        log.debug("restore {}", ctx.getId());
+
+        boolean rollback = false;
         try {
-            final ResellerAdmin adm = getData(new ResellerAdmin[] { new ResellerAdmin(creds.getLogin(), creds.getPassword()) })[0];
-            if (ctx.getId() == null) {
-                throw new InvalidDataException("ContextID must not be null");
-            }
             oxcon = cache.getWriteConnectionForConfigDB();
             oxcon.setAutoCommit(false);
-            prep = oxcon.prepareStatement("INSERT INTO context2subadmin (sid,cid) VALUES(?,?)");
-            prep.setInt(1, adm.getId());
-            prep.setInt(2, ctx.getId());
-            prep.executeUpdate();
+            rollback = true;
 
-            oxcon.commit();
+            if (subadmin > 0) {
+                prep = oxcon.prepareStatement("INSERT INTO context2subadmin (sid,cid) VALUES(?,?)");
+                prep.setInt(1, subadmin);
+                prep.setInt(2, ctx.getId().intValue());
+                prep.executeUpdate();
+                Databases.closeSQLStuff(prep);
+                prep = null;
+            }
+
+            if (restrictions != null) {
+                for (Restriction r : restrictions) {
+                    prep = oxcon.prepareStatement("INSERT INTO context_restrictions (cid,rid,value) VALUES (?,?,?)");
+                    prep.setInt(1, ctx.getId().intValue());
+                    prep.setInt(2, r.getId().intValue());
+                    prep.setString(3, r.getValue());
+                    prep.executeUpdate();
+                    Databases.closeSQLStuff(prep);
+                    prep = null;
+                }
+            }
+
+            if (customFields != null) {
+                for (CustomField customField : customFields) {
+                    prep = oxcon.prepareStatement("INSERT INTO context_customfields (cid, customid, createTimestamp, modifyTimestamp) VALUES (?,?, ?, ?)");
+                    prep.setInt(1, ctx.getId().intValue());
+                    prep.setString(2, customField.getCustomId());
+                    prep.setLong(3, customField.getCreateTimestamp());
+                    prep.setLong(4, customField.getModifyTimestamp());
+                    prep.executeUpdate();
+                    Databases.closeSQLStuff(prep);
+                    prep = null;
+                }
+            }
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
-            doRollback(oxcon);
             throw AdminCache.parseDataTruncation(dt);
         } catch (final RuntimeException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw e;
         } catch (final PoolException e) {
             log.error("", e);
@@ -592,22 +639,67 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(oxcon);
-            throw new StorageException(e.getMessage());
-        } catch (final InvalidDataException e) {
-            log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(oxcon);
+            }
             cache.closeWriteConfigDBSqlStuff(oxcon, prep);
         }
     }
 
     @Override
-    public void unownContextFromAdmin(final Context ctx, final Credentials creds) throws StorageException {
+    public void ownContextToAdmin(final Context ctx, final Credentials creds) throws StorageException {
+        Connection oxcon = null;
+        PreparedStatement prep = null;
+
+        log.debug("ownContext {} to admin {}", ctx.getId(), creds.getLogin());
+
+        boolean rollback = false;
         try {
             final ResellerAdmin adm = getData(new ResellerAdmin[] { new ResellerAdmin(creds.getLogin(), creds.getPassword()) })[0];
-            unownContextFromAdmin(ctx, adm);
+            if (ctx.getId() == null) {
+                throw new InvalidDataException("ContextID must not be null");
+            }
+            oxcon = cache.getWriteConnectionForConfigDB();
+            oxcon.setAutoCommit(false);
+            rollback = true;
+            prep = oxcon.prepareStatement("INSERT INTO context2subadmin (sid,cid) VALUES(?,?)");
+            prep.setInt(1, adm.getId());
+            prep.setInt(2, ctx.getId());
+            prep.executeUpdate();
+
+            oxcon.commit();
+            rollback = false;
+        } catch (final DataTruncation dt) {
+            log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
+            throw AdminCache.parseDataTruncation(dt);
+        } catch (final RuntimeException e) {
+            log.error("", e);
+            throw e;
+        } catch (final PoolException e) {
+            log.error("", e);
+            // no Rollback needed as the connection is null at this moment
+            throw new StorageException(e.getMessage());
+        } catch (final SQLException e) {
+            log.error("", e);
+            throw new StorageException(e.getMessage());
+        } catch (final InvalidDataException e) {
+            log.error("", e);
+            throw new StorageException(e.getMessage());
+        } finally {
+            if (rollback) {
+                doRollback(oxcon);
+            }
+            cache.closeWriteConfigDBSqlStuff(oxcon, prep);
+        }
+    }
+
+    @Override
+    public int unownContextFromAdmin(final Context ctx, final Credentials creds) throws StorageException {
+        try {
+            final ResellerAdmin adm = getData(new ResellerAdmin[] { new ResellerAdmin(creds.getLogin(), creds.getPassword()) })[0];
+            return unownContextFromAdmin(ctx, adm);
         } catch (final RuntimeException e) {
             log.error("", e);
             throw e;
@@ -615,9 +707,13 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
     }
 
     @Override
-    public void unownContextFromAdmin(final Context ctx, final ResellerAdmin adm) throws StorageException {
+    public int unownContextFromAdmin(final Context ctx, final ResellerAdmin adm) throws StorageException {
         Connection oxcon = null;
         PreparedStatement prep = null;
+
+        log.debug("unownContext {} from admin {}", ctx.getId(), adm.getName());
+
+        boolean rollback = false;
         try {
             if (adm.getId() == null) {
                 throw new InvalidDataException("ResellerAdminID must not be null");
@@ -627,19 +723,24 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             }
             oxcon = cache.getWriteConnectionForConfigDB();
             oxcon.setAutoCommit(false);
+            rollback = true;
+
+            int subadminId = adm.getId().intValue();
+
             prep = oxcon.prepareStatement("DELETE FROM context2subadmin WHERE sid=? AND cid=?");
-            prep.setInt(1, adm.getId());
-            prep.setInt(2, ctx.getId());
+            prep.setInt(1, subadminId);
+            prep.setInt(2, ctx.getId().intValue());
             prep.executeUpdate();
 
             oxcon.commit();
+            rollback = false;
+
+            return subadminId;
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
-            doRollback(oxcon);
             throw AdminCache.parseDataTruncation(dt);
         } catch (final RuntimeException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw e;
         } catch (final PoolException e) {
             log.error("", e);
@@ -647,13 +748,14 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final InvalidDataException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(oxcon);
+            }
             cache.closeWriteConfigDBSqlStuff(oxcon, prep);
         }
     }
@@ -663,6 +765,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection oxcon = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
+        log.debug("getContextOwner from context {}", ctx.getId());
         try {
             if (ctx.getId() == null) {
                 throw new InvalidDataException("ContextID must not be null");
@@ -700,6 +803,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection oxcon = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
+        log.debug("ownsContext ctx={}  admin={}", null == ctx ? null : ctx.getId(), admid);
         try {
             oxcon = cache.getReadConnectionForConfigDB();
             if (ctx == null) {
@@ -744,6 +848,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection oxcon = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
+        log.debug("ownsContextOrIsPidOfOwner ctx={}  admin={}", null == ctx ? null : ctx.getId(), admid);
         try {
             oxcon = cache.getReadConnectionForConfigDB();
             if (ctx == null) {
@@ -800,6 +905,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection oxcon = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
+        log.debug("checkOwnsContextAndSetSid ctx={}  admin={}", null == ctx ? null : ctx.getId(), creds.getLogin());
         try {
             final ResellerAdmin adm = getData(new ResellerAdmin[] { new ResellerAdmin(creds.getLogin(), creds.getPassword()) })[0];
             oxcon = cache.getReadConnectionForConfigDB();
@@ -856,6 +962,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
+        log.debug("listRestrictions search_pattern={}", search_pattern);
         try {
             con = cache.getReadConnectionForConfigDB();
             final String search_patterntmp = search_pattern.replace('*', '%');
@@ -1211,6 +1318,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public void checkPerSubadminRestrictions(final Credentials creds, final UserModuleAccess access, boolean contextAdmin, final String... restriction_types) throws StorageException {
+        log.debug("checkPerSubadminRestrictions");
         final ResellerAdmin adm = getData(new ResellerAdmin[] { new ResellerAdmin(creds.getLogin(), creds.getPassword()) })[0];
         HashSet<Restriction> restrictions = OXResellerTools.array2HashSet(adm.getRestrictions());
         // default is: not allowed to create SUBADMINS
@@ -1296,6 +1404,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public void checkPerContextRestrictions(final Context ctx, final UserModuleAccess access, boolean contextAdmin, final String... restriction_types) throws StorageException {
+        log.debug("checkPerContextRestrictions");
         Connection con = null;
 
         try {
@@ -1403,52 +1512,74 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      * com.openexchange.admin.rmi.dataobjects.Context)
      */
     @Override
-    public void applyRestrictionsToContext(final Restriction[] restrictions, final Context ctx) throws StorageException {
+    public Restriction[] applyRestrictionsToContext(final Restriction[] restrictions, final Context ctx) throws StorageException {
+        log.debug("applyRestrictionsToContext {}", ctx);
         Connection oxcon = null;
         PreparedStatement prep = null;
+        ResultSet rs = null;
 
+        List<Restriction> dropped = null;
+
+        boolean rollback = false;
         try {
             oxcon = cache.getWriteConnectionForConfigDB();
             oxcon.setAutoCommit(false);
+            rollback = true;
+
+            final int cid = ctx.getId().intValue();
+            prep = oxcon.prepareStatement("SELECT rid, value FROM context_restrictions WHERE cid=?");
+            prep.setInt(1, cid);
+            rs = prep.executeQuery();
+            if (rs.next()) {
+                dropped = new LinkedList<Restriction>();
+                do {
+                    dropped.add(new Restriction(Integer.valueOf(rs.getInt(1)), null, rs.getString(2)));
+                } while (rs.next());
+            }
+            Databases.closeSQLStuff(rs, prep);
+            rs = null;
+            prep = null;
 
-            final int cid = ctx.getId();
             prep = oxcon.prepareStatement("DELETE FROM context_restrictions WHERE cid=?");
             prep.setInt(1, cid);
             prep.executeUpdate();
-            prep.close();
+            Databases.closeSQLStuff(prep);
+            prep = null;
+
             if (restrictions != null) {
-                for(final Restriction r : restrictions) {
+                for (final Restriction r : restrictions) {
                     prep = oxcon.prepareStatement("INSERT INTO context_restrictions (cid,rid,value) VALUES (?,?,?)");
                     prep.setInt(1, cid);
-                    prep.setInt(2, r.getId());
+                    prep.setInt(2, r.getId().intValue());
                     prep.setString(3, r.getValue());
                     prep.executeUpdate();
-                    prep.close();
+                    Databases.closeSQLStuff(prep);
+                    prep = null;
                 }
             }
 
             oxcon.commit();
+            rollback = false;
         } catch (final DataTruncation dt) {
             log.error(AdminCache.DATA_TRUNCATION_ERROR_MSG, dt);
-            doRollback(oxcon);
             throw AdminCache.parseDataTruncation(dt);
         } catch (final RuntimeException e) {
             log.error("", e);
-            if (null != oxcon) {
-                doRollback(oxcon);
-            }
             throw e;
         } catch (final PoolException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(oxcon);
             throw new StorageException(e.getMessage());
         } finally {
-            cache.closeWriteConfigDBSqlStuff(oxcon, prep);
+            if (rollback) {
+                doRollback(oxcon);
+            }
+            cache.closeWriteConfigDBSqlStuff(oxcon, prep, rs);
         }
+
+        return dropped == null ? new Restriction[0] : dropped.toArray(new Restriction[dropped.size()]);
     }
 
     /*
@@ -1459,6 +1590,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public Restriction[] getRestrictionsFromContext(final Context ctx) throws StorageException {
+        log.debug("getRestrictionsFromContext {}", ctx);
         Connection con = null;
         try {
             con = cache.getReadConnectionForConfigDB();
@@ -1507,11 +1639,16 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public void initDatabaseRestrictions() throws StorageException {
+        log.debug("initDatabaseRestrictions");
         Connection con = null;
         PreparedStatement prep = null;
+
+        boolean rollback = false;
         try {
             con = cache.getWriteConnectionForConfigDB();
             con.setAutoCommit(false);
+            rollback = true;
+
             for (final String res : Restriction.ALL_RESTRICTIONS) {
                 final int rid = IDGenerator.getId(con);
                 prep = con.prepareStatement("INSERT INTO restrictions (rid,name) VALUES (?,?)");
@@ -1533,24 +1670,25 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
                     prep.close();
                 }
             }
+
             con.commit();
+            rollback = true;
         } catch (final PoolException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } catch (final ClassNotFoundException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } catch (final OXGenericException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(con);
+            }
             cache.closeWriteConfigDBSqlStuff(con, prep);
         }
     }
@@ -1561,23 +1699,30 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public void removeDatabaseRestrictions() throws StorageException {
+        log.debug("removeDatabaseRestrictions");
         Connection con = null;
         PreparedStatement prep = null;
+        boolean rollback = false;
         try {
             con = cache.getWriteConnectionForConfigDB();
             con.setAutoCommit(false);
+            rollback = true;
+
             prep = con.prepareStatement("DELETE FROM restrictions");
             prep.executeUpdate();
+
             con.commit();
+            rollback = false;
         } catch (final PoolException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(con);
+            }
             cache.closeWriteConfigDBSqlStuff(con, prep);
         }
     }
@@ -1587,6 +1732,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public String getCustomId(final Context ctx) throws StorageException {
+        log.debug("getCustomId from context {}", ctx);
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
@@ -1615,6 +1761,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public void writeCustomId(final Context ctx) throws StorageException {
+        log.debug("writeCustomId to context {}", ctx);
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
@@ -1655,16 +1802,36 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
     }
 
     @Override
-    public void deleteCustomFields(final Context ctx) throws StorageException {
+    public CustomField[] deleteCustomFields(final Context ctx) throws StorageException {
+        log.debug("deleteCustomFields from context {}", ctx);
         Connection con = null;
         PreparedStatement prep = null;
-        final ResultSet rs = null;
+        ResultSet rs = null;
         try {
             con = cache.getWriteConnectionForConfigDB();
+
+            prep = con.prepareStatement("SELECT customid, createTimestamp, modifyTimestamp FROM context_customfields WHERE cid=?");
+            prep.setInt(1, ctx.getId().intValue());
+            rs = prep.executeQuery();
+            if (!rs.next()) {
+                return new CustomField[0];
+            }
+
+            List<CustomField> dropped = new LinkedList<CustomField>();
+            do {
+                dropped.add(new CustomField(rs.getString(1), rs.getLong(2), rs.getLong(3)));
+            } while (rs.next());
+            Databases.closeSQLStuff(rs, prep);
+            rs = null;
+            prep = null;
+
             prep = con.prepareStatement("DELETE FROM context_customfields WHERE cid=?");
-            prep.setInt(1, ctx.getId());
+            prep.setInt(1, ctx.getId().intValue());
             prep.executeUpdate();
-            prep.close();
+            Databases.closeSQLStuff(prep);
+            prep = null;
+
+            return dropped.toArray(new CustomField[dropped.size()]);
         } catch (final PoolException e) {
             log.error("", e);
             throw new StorageException(e.getMessage());
@@ -1678,6 +1845,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
 
     @Override
     public void generateCreateTimestamp(final Context ctx) throws StorageException {
+        log.debug("generateCreateTimestamp for context {}", ctx);
         Connection con = null;
         PreparedStatement prep = null;
         final ResultSet rs = null;
@@ -1703,6 +1871,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
 
     @Override
     public void updateModifyTimestamp(final Context ctx) throws StorageException {
+        log.debug("updateModifyTimestamp for context {}", ctx);
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
@@ -1711,11 +1880,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             prep = con.prepareStatement("SELECT * FROM context_customfields WHERE cid=?");
             prep.setInt(1, ctx.getId());
             rs = prep.executeQuery();
-            if( ! rs.next() ) {
-                generateCreateTimestamp(ctx);
-                prep.close();
-                rs.close();
-            } else {
+            if( rs.next() ) {
                 prep.close();
                 rs.close();
                 prep = con.prepareStatement("UPDATE context_customfields SET modifyTimestamp=? WHERE cid=?");
@@ -1738,18 +1903,26 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
 
     private void removeRestriction(final Connection con, final String name) throws SQLException {
         final PreparedStatement prep = con.prepareStatement("DELETE FROM restrictions WHERE name = ?");
-        prep.setString(1, name);
-        prep.executeUpdate();
-        prep.close();
+        try {
+            prep.setString(1, name);
+            prep.executeUpdate();
+            prep.close();
+        } finally {
+            DBUtils.closeSQLStuff(prep);
+        }
     }
 
     private void addRestriction(final Connection con, final String name) throws SQLException {
         final int rid = IDGenerator.getId(con);
         final PreparedStatement prep = con.prepareStatement("INSERT INTO restrictions (rid,name) VALUES (?,?)");
-        prep.setInt(1, rid);
-        prep.setString(2, name);
-        prep.executeUpdate();
-        prep.close();
+        try {
+            prep.setInt(1, rid);
+            prep.setString(2, name);
+            prep.executeUpdate();
+            prep.close();
+        } finally {
+            DBUtils.closeSQLStuff(prep);
+        }
     }
 
     /*
@@ -1758,9 +1931,11 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
      */
     @Override
     public void updateModuleAccessRestrictions() throws StorageException, OXResellerException {
+        log.debug("updateModuleAccessRestrictions");
         Connection con = null;
         PreparedStatement prep = null;
         ResultSet rs = null;
+        boolean rollback = false;
         try {
             con = cache.getWriteConnectionForConfigDB();
             cache.initAccessCombinations();
@@ -1804,6 +1979,7 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             final Map<String, Restriction> curCombinations = listRestrictions("*");
 
             con.setAutoCommit(false);
+            rollback = true;
             for(final String cname : newCombinations.keySet()) {
                 final String percontext  = Restriction.MAX_USER_PER_CONTEXT_BY_MODULEACCESS_PREFIX+cname;
                 final String persubadmin = Restriction.MAX_OVERALL_USER_PER_SUBADMIN_BY_MODULEACCESS_PREFIX+cname;
@@ -1827,37 +2003,41 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             }
 
             con.commit();
+            rollback = false;
         } catch (final PoolException e) {
             log.error("", e);
             // no Rollback needed as the connection is null at this moment
             throw new StorageException(e.getMessage());
         } catch (final SQLException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } catch (final ClassNotFoundException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } catch (final OXGenericException e) {
             log.error("", e);
-            doRollback(con);
             throw new StorageException(e.getMessage());
         } finally {
+            if (rollback) {
+                doRollback(con);
+            }
             cache.closeWriteConfigDBSqlStuff(con, prep, rs);
         }
     }
 
     private void doRollback(final Connection con) {
-        try {
-            con.rollback();
-        } catch (final SQLException e2) {
-            log.error("Error doing rollback", e2);
+        if (null != con) {
+            try {
+                con.rollback();
+            } catch (final SQLException e2) {
+                log.error("Error doing rollback", e2);
+            }
         }
     }
 
     @Override
     public void updateRestrictions() throws StorageException, OXResellerException {
+        log.debug("updateRestrictions");
         Connection con = null;
         PreparedStatement prep = null;
 
@@ -1869,9 +2049,12 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
             }
         }
         if( missingRestrictions.size() > 0 ) {
+            boolean rollback = false;
             try {
                 con = cache.getWriteConnectionForConfigDB();
                 con.setAutoCommit(false);
+                rollback = true;
+
                 for (final String res : missingRestrictions) {
                     final int rid = IDGenerator.getId(con);
                     prep = con.prepareStatement("INSERT INTO restrictions (rid,name) VALUES (?,?)");
@@ -1880,16 +2063,19 @@ public final class OXResellerMySQLStorage extends OXResellerSQLStorage {
                     prep.executeUpdate();
                     prep.close();
                 }
+
                 con.commit();
+                rollback = false;
             } catch (final PoolException e) {
                 log.error("", e);
-                doRollback(con);
                 throw new StorageException(e.getMessage());
             } catch (final SQLException e) {
                 log.error("", e);
-                doRollback(con);
                 throw new StorageException(e.getMessage());
             } finally {
+                if (rollback) {
+                    doRollback(con);
+                }
                 cache.closeWriteConfigDBSqlStuff(con, prep);
             }
         }
